/*
** Module   : ToggleBox.java
** Abstract : toggle box implementation
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20140610 Initial implementation.
** 002 CA  20140805 Added generics to force the abstract widget classes to not use implementation-
**                  specific code: the abstract widgets will have access only to the base APIs
**                  provided by the OutputManager.
** 003 HC  20140911 GUI coordinate support - data types refactoring.
** 004 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 005 HC  20141021 Implemented window move/position support.
** 006 HC  20150323 Removed business logic from config classes.
** 007 VIG 20150506 Extracted check method.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * An implementation of a "toggle box" with a text label.
 */
public abstract class ToggleBox<O extends OutputManager<?>>
extends AbstractButton<O,ToggleBoxConfig>
implements DataContainer
{
   /**
    * This constructor creates a toggle-box with a given text label.
    * 
    * @param   id
    *          The widget's ID.
    * @param   owner
    *          The parent container, frame or dialog which will include this toggle-box.
    * @param   text
    *          The string to be used as the toggle-box text.
    */
   public ToggleBox(WidgetId id, Container<O> owner, String text)
   {
      disable(); // disabled by default
      setTextLabel(text);
      
      config = ConfigManager.getInstance().resolveWidgetConfig(id, ToggleBoxConfig.class);
      
      config.name = text; // direct access
      config.widthChars = width(); // direct access
      config.help = null;
      addFocusListener(this);
   }

   /**
    * Special constructor used to restore component using given configuration.
    * <p>
    * Call {@link #initialize} to finish the widget initialization.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public ToggleBox(WidgetId id, ToggleBoxConfig cfg)
   {
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, ToggleBoxConfig cfg)
   {
      disable(); // disabled by default
      setTextLabel("");
      // this constant is defined during testing.
      config = (ToggleBoxConfig) ConfigManager.getInstance().resolveWidgetConfig(id, cfg);
      ConfigManager.getInstance().replaceConfig(config, cfg);

      if (cfg.label != null)
         setText(cfg.label);
      
      addFocusListener(this);
   }

   // DataContainer methods implementations start
   /**
    * Get label text.
    * 
    * @return  label text.
    */
   @Override
   public String getText()
   {
      return textLabel();
   }

   /**
    * Getting the BaseDataType object representing the current value.
    *
    * @return  The value representing the current widget value.
    */
   @Override
   public BaseDataType getValue()
   {
      return new logical(config.checked);
   }

   /**
    * Getting the modification state of the widget.
    * 
    * @return  <code>true</code> if the widget value was modified,
    *          <code>false</code> otherwise.
    */
   @Override
   public boolean isModified()
   {
      return config.modified;
   }

   /**
    * Sets the value of the toggle-box (the checked state) based on a
    * BaseDataType value.
    * 
    * @param    value
    *           The BaseDataType value of the logical value for toggle-box.
    */
   @Override
   public void setValue(BaseDataType value)
   {
      if (value instanceof logical)
      {
         config.checked = ((logical)value).booleanValue();
      }
   }
   // DataContainer methods implementations end
   
   /**
    * Calculate the width of this widget.
    *
    * @return    The width in characters.
    */
   @Override
   public double width()
   {
      // safe, comparison of scaled doubles
      return config.widthChars > 0 ? config.widthChars : textLabel().length() + 3;
   }
   
   /**
    * Calculate the height of this widget.
    *
    * @return    The height in rows.
    */
   @Override
   public double height()
   {
      // safe, comparison of scaled doubles
      return config.heightChars > 0 ? config.heightChars : 1;
   }
   
   /**
    * Processes key events occurring on this object by dispatching them
    * to any registered KeyListener objects.
    *
    * @param    keyEvent
    *           The event to be handled.
    */
   @Override
   public void processKeyEvent(KeyInput keyEvent)
   {
      super.processKeyEventCommon(keyEvent);
   }

   /**
    * Set the new text for the component.
    * 
    * @param   text
    *          New text value.
    */
   @Override
   public void setText(String text)
   {
      setTextLabel(text);

      repaint();
      
      if (config.autoResize)
         config.widthChars = textLabel().length() + 3; // direct access
   }

   /**
    * Process basic events - TAB, BACK-TAB and ENTER key event.
    * 
    * @param   event
    *          Key event to process.
    */
   @Override
   protected void handleBasicEvents(KeyInput event)
   {
      super.processKeyEvent(event);

      if (event.isConsumed())
      {
         return;
      }

      int key = event.keyCode();
      if (key == '\t')
      {
         parent().nextFocus();
         return;
      }
      else if (key == Key.VK_BACK_TAB)
      {
         parent().prevFocus();
         return;
      }

      if (super.isEnabled() && (key == Key.VK_ENTER || key == Key.VK_SPACE))
      {
         check();
         // eat the original event
         event.consume();
      }
   }

   /**
    * Inverse checked state and post VALUE-CHANGED event.
    * 
    * @param event
    *        
    */
   protected void check()
   {
      config.checked = !config.checked;
      ConfigHelper.setModified(config, true);
      refresh();
      // Generate VALUE_CHANGED event
      KeyInput valueChangedEvent = new KeyInput(this, EventType.KEY_TYPED,
                                                Keyboard.SE_VALUE_CHANGED);
      EventManager.postEvent(valueChangedEvent);
   }
}
